@@ -2,7 +2,7 @@
 // tcp.cpp
 // ~~~~~~~
 //
-// Copyright (c) 2003-2022 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2023 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -239,6 +239,7 @@ void test()
     archetypes::gettable_socket_option<int> gettable_socket_option2;
     archetypes::gettable_socket_option<double> gettable_socket_option3;
     archetypes::io_control_command io_control_command;
+    archetypes::immediate_handler immediate;
     archetypes::lazy_handler lazy;
     boost::system::error_code ec;
 
@@ -346,6 +347,8 @@ void test()
         connect_handler());
     socket1.async_connect(ip::tcp::endpoint(ip::tcp::v6(), 0),
         connect_handler());
+    socket1.async_connect(ip::tcp::endpoint(ip::tcp::v4(), 0), immediate);
+    socket1.async_connect(ip::tcp::endpoint(ip::tcp::v6(), 0), immediate);
     int i1 = socket1.async_connect(ip::tcp::endpoint(ip::tcp::v4(), 0), lazy);
     (void)i1;
     int i2 = socket1.async_connect(ip::tcp::endpoint(ip::tcp::v6(), 0), lazy);
@@ -395,6 +398,7 @@ void test()
     socket1.wait(socket_base::wait_write, ec);
 
     socket1.async_wait(socket_base::wait_read, wait_handler());
+    socket1.async_wait(socket_base::wait_read, immediate);
     int i3 = socket1.async_wait(socket_base::wait_write, lazy);
     (void)i3;
 
@@ -426,6 +430,16 @@ void test()
     socket1.async_send(mutable_buffers, in_flags, send_handler());
     socket1.async_send(const_buffers, in_flags, send_handler());
     socket1.async_send(null_buffers(), in_flags, send_handler());
+    socket1.async_send(buffer(mutable_char_buffer), immediate);
+    socket1.async_send(buffer(const_char_buffer), immediate);
+    socket1.async_send(mutable_buffers, immediate);
+    socket1.async_send(const_buffers, immediate);
+    socket1.async_send(null_buffers(), immediate);
+    socket1.async_send(buffer(mutable_char_buffer), in_flags, immediate);
+    socket1.async_send(buffer(const_char_buffer), in_flags, immediate);
+    socket1.async_send(mutable_buffers, in_flags, immediate);
+    socket1.async_send(const_buffers, in_flags, immediate);
+    socket1.async_send(null_buffers(), in_flags, immediate);
     int i4 = socket1.async_send(buffer(mutable_char_buffer), lazy);
     (void)i4;
     int i5 = socket1.async_send(buffer(const_char_buffer), lazy);
@@ -464,6 +478,12 @@ void test()
         receive_handler());
     socket1.async_receive(mutable_buffers, in_flags, receive_handler());
     socket1.async_receive(null_buffers(), in_flags, receive_handler());
+    socket1.async_receive(buffer(mutable_char_buffer), immediate);
+    socket1.async_receive(mutable_buffers, immediate);
+    socket1.async_receive(null_buffers(), immediate);
+    socket1.async_receive(buffer(mutable_char_buffer), in_flags, immediate);
+    socket1.async_receive(mutable_buffers, in_flags, immediate);
+    socket1.async_receive(null_buffers(), in_flags, immediate);
     int i14 = socket1.async_receive(buffer(mutable_char_buffer), lazy);
     (void)i14;
     int i15 = socket1.async_receive(mutable_buffers, lazy);
@@ -494,6 +514,11 @@ void test()
     socket1.async_write_some(mutable_buffers, write_some_handler());
     socket1.async_write_some(const_buffers, write_some_handler());
     socket1.async_write_some(null_buffers(), write_some_handler());
+    socket1.async_write_some(buffer(mutable_char_buffer), immediate);
+    socket1.async_write_some(buffer(const_char_buffer), immediate);
+    socket1.async_write_some(mutable_buffers, immediate);
+    socket1.async_write_some(const_buffers, immediate);
+    socket1.async_write_some(null_buffers(), immediate);
     int i20 = socket1.async_write_some(buffer(mutable_char_buffer), lazy);
     (void)i20;
     int i21 = socket1.async_write_some(buffer(const_char_buffer), lazy);
@@ -515,6 +540,9 @@ void test()
     socket1.async_read_some(buffer(mutable_char_buffer), read_some_handler());
     socket1.async_read_some(mutable_buffers, read_some_handler());
     socket1.async_read_some(null_buffers(), read_some_handler());
+    socket1.async_read_some(buffer(mutable_char_buffer), immediate);
+    socket1.async_read_some(mutable_buffers, immediate);
+    socket1.async_read_some(null_buffers(), immediate);
     int i25 = socket1.async_read_some(buffer(mutable_char_buffer), lazy);
     (void)i25;
     int i26 = socket1.async_read_some(mutable_buffers, lazy);
@@ -767,6 +795,7 @@ void test()
     archetypes::gettable_socket_option<int> gettable_socket_option2;
     archetypes::gettable_socket_option<double> gettable_socket_option3;
     archetypes::io_control_command io_control_command;
+    archetypes::immediate_handler immediate;
     archetypes::lazy_handler lazy;
     boost::system::error_code ec;
 
@@ -883,6 +912,7 @@ void test()
     acceptor1.wait(socket_base::wait_write, ec);
 
     acceptor1.async_wait(socket_base::wait_read, wait_handler());
+    acceptor1.async_wait(socket_base::wait_read, immediate);
     int i1 = acceptor1.async_wait(socket_base::wait_write, lazy);
     (void)i1;
 
@@ -914,6 +944,8 @@ void test()
 
     acceptor1.async_accept(peer_socket1, accept_handler());
     acceptor1.async_accept(peer_socket1, peer_endpoint, accept_handler());
+    acceptor1.async_accept(peer_socket1, immediate);
+    acceptor1.async_accept(peer_socket1, peer_endpoint, immediate);
     int i2 = acceptor1.async_accept(peer_socket1, lazy);
     (void)i2;
     int i3 = acceptor1.async_accept(peer_socket1, peer_endpoint, lazy);
@@ -921,6 +953,8 @@ void test()
 
     acceptor1.async_accept(peer_socket2, accept_handler());
     acceptor1.async_accept(peer_socket2, peer_endpoint, accept_handler());
+    acceptor1.async_accept(peer_socket2, immediate);
+    acceptor1.async_accept(peer_socket2, peer_endpoint, immediate);
     int i4 = acceptor1.async_accept(peer_socket2, lazy);
     (void)i4;
     int i5 = acceptor1.async_accept(peer_socket2, peer_endpoint, lazy);
@@ -935,6 +969,12 @@ void test()
     acceptor1.async_accept(ioc, peer_endpoint, move_accept_handler());
     acceptor1.async_accept(ioc_ex, peer_endpoint, move_accept_handler());
     acceptor1.async_accept(ioc_ex, peer_endpoint, move_accept_ioc_handler());
+    acceptor1.async_accept(immediate);
+    acceptor1.async_accept(ioc, immediate);
+    acceptor1.async_accept(ioc_ex, immediate);
+    acceptor1.async_accept(peer_endpoint, immediate);
+    acceptor1.async_accept(ioc, peer_endpoint, immediate);
+    acceptor1.async_accept(ioc_ex, peer_endpoint, immediate);
 #endif // defined(BOOST_ASIO_HAS_MOVE)
   }
   catch (std::exception&)
