@@ -23,9 +23,12 @@
 #endif
 #include <boost/assert.hpp>
 #include <boost/system/error_category.hpp>
+#include <boost/iterator/is_iterator.hpp>
 #include <boost/type_traits/declval.hpp>
 #include <boost/type_traits/remove_cv.hpp>
-#if defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+#include <boost/type_traits/integral_constant.hpp>
+#include <boost/type_traits/conjunction.hpp>
+#if defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 #include <boost/type_traits/disjunction.hpp>
 #include <boost/core/enable_if.hpp>
 #endif
@@ -311,78 +314,87 @@ struct path_source_traits< directory_entry >
 
 //! The trait tests if the type is a known path Source tag
 template< typename Tag >
-struct is_known_path_source_tag
+struct is_known_path_source_tag :
+    public boost::true_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = true;
 };
 
 template< >
-struct is_known_path_source_tag< unknown_type_tag >
+struct is_known_path_source_tag< unknown_type_tag > :
+    public boost::false_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = false;
 };
 
 //! The trait tests if the type is compatible with path Source requirements
 template< typename T >
 struct is_path_source :
-    public is_known_path_source_tag< typename path_source_traits< T >::tag_type >
+    public is_known_path_source_tag< typename path_source_traits< T >::tag_type >::type
 {
 };
 
 
 //! The trait indicates whether the type is a path Source that is natively supported by path::string_type as the source for construction/assignment/appending
 template< typename T >
-struct is_native_path_source
+struct is_native_path_source :
+    public boost::integral_constant< bool, path_source_traits< T >::is_native >
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = path_source_traits< T >::is_native;
 };
 
 
 //! The trait indicates whether the type is one of the supported path character types
 template< typename T >
-struct is_path_char_type
+struct is_path_char_type :
+    public boost::false_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = false;
 };
 
 template< >
-struct is_path_char_type< char >
+struct is_path_char_type< char > :
+    public boost::true_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = true;
 };
 
 template< >
-struct is_path_char_type< wchar_t >
+struct is_path_char_type< wchar_t > :
+    public boost::true_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = true;
 };
 
 
+template< typename Iterator >
+struct is_iterator_to_path_chars :
+    public is_path_char_type< typename std::iterator_traits< Iterator >::value_type >::type
+{
+};
+
 //! The trait indicates whether the type is an iterator over a sequence of path characters
 template< typename Iterator >
 struct is_path_source_iterator :
-    public is_path_char_type< typename std::iterator_traits< Iterator >::value_type >
+    public boost::conjunction<
+        boost::iterators::is_iterator< Iterator >,
+        is_iterator_to_path_chars< Iterator >
+    >::type
 {
 };
 
 
 //! The trait indicates whether the type is a pointer to a sequence of native path characters
 template< typename T >
-struct is_native_char_ptr
+struct is_native_char_ptr :
+    public boost::false_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = false;
 };
 
 template< >
-struct is_native_char_ptr< path_native_char_type* >
+struct is_native_char_ptr< path_native_char_type* > :
+    public boost::true_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = true;
 };
 
 template< >
-struct is_native_char_ptr< const path_native_char_type* >
+struct is_native_char_ptr< const path_native_char_type* > :
+    public boost::true_type
 {
-    static BOOST_CONSTEXPR_OR_CONST bool value = true;
 };
 
 
@@ -398,37 +410,37 @@ void convert(const wchar_t* from, const wchar_t* from_end, std::string& to, cons
 //  Source dispatch  -----------------------------------------------------------------//
 
 template< typename Source, typename Callback >
-inline void dispatch(Source const& source, Callback cb, const codecvt_type* cvt = NULL);
+typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt = NULL);
 
 template< typename Callback >
-BOOST_FORCEINLINE void dispatch(const char* source, Callback cb, const codecvt_type* cvt, ntcts_type_tag)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(const char* source, Callback cb, const codecvt_type* cvt, ntcts_type_tag)
 {
-    cb(source, source + std::strlen(source), cvt);
+    return cb(source, source + std::strlen(source), cvt);
 }
 
 template< typename Callback >
-BOOST_FORCEINLINE void dispatch(const wchar_t* source, Callback cb, const codecvt_type* cvt, ntcts_type_tag)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(const wchar_t* source, Callback cb, const codecvt_type* cvt, ntcts_type_tag)
 {
-    cb(source, source + std::wcslen(source), cvt);
+    return cb(source, source + std::wcslen(source), cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch(Source const& source, Callback cb, const codecvt_type* cvt, string_class_tag)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt, string_class_tag)
 {
-    cb(source.data(), source.data() + source.size(), cvt);
+    return cb(source.data(), source.data() + source.size(), cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch(Source const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
 {
     std::basic_string< typename Source::value_type > src(source.begin(), source.end());
-    cb(src.data(), src.data() + src.size(), cvt);
+    return cb(src.data(), src.data() + src.size(), cvt);
 }
 
 #if !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 
 template< typename Callback >
-BOOST_FORCEINLINE void dispatch(std::vector< char > const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(std::vector< char > const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
 {
     const char* data = NULL, *data_end = NULL;
     if (!source.empty())
@@ -436,11 +448,11 @@ BOOST_FORCEINLINE void dispatch(std::vector< char > const& source, Callback cb,
         data = &source[0];
         data_end = data + source.size();
     }
-    cb(data, data_end, cvt);
+    return cb(data, data_end, cvt);
 }
 
 template< typename Callback >
-BOOST_FORCEINLINE void dispatch(std::vector< wchar_t > const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(std::vector< wchar_t > const& source, Callback cb, const codecvt_type* cvt, range_type_tag)
 {
     const wchar_t* data = NULL, *data_end = NULL;
     if (!source.empty())
@@ -448,19 +460,19 @@ BOOST_FORCEINLINE void dispatch(std::vector< wchar_t > const& source, Callback c
         data = &source[0];
         data_end = data + source.size();
     }
-    cb(data, data_end, cvt);
+    return cb(data, data_end, cvt);
 }
 
 #endif // !defined(BOOST_FILESYSTEM_NO_DEPRECATED) && BOOST_FILESYSTEM_VERSION < 4
 
 // Defined in directory.hpp to avoid circular header dependencies
 template< typename Callback >
-void dispatch(directory_entry const& de, Callback cb, const codecvt_type* cvt, directory_entry_tag);
+typename Callback::result_type dispatch(directory_entry const& de, Callback cb, const codecvt_type* cvt, directory_entry_tag);
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch(Source const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch(Source const& source, Callback cb, const codecvt_type* cvt)
 {
-    path_traits::dispatch(source, cb, cvt,
+    return path_traits::dispatch(source, cb, cvt,
         typename path_traits::path_source_traits< typename boost::remove_cv< Source >::type >::tag_type());
 }
 
@@ -468,34 +480,40 @@ BOOST_FORCEINLINE void dispatch(Source const& source, Callback cb, const codecvt
 typedef char yes_type;
 struct no_type { char buf[2]; };
 
-// Note: The obscure naming of the _check* functions below is a workaround for an MSVC-9.0 bug, which looks up the function outside the class scope
+#if !defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 
-#if !defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+namespace is_convertible_to_path_source_impl {
+
+yes_type check(const char*);
+yes_type check(const wchar_t*);
+yes_type check(std::string const&);
+yes_type check(std::wstring const&);
+yes_type check(boost::container::basic_string< char, std::char_traits< char >, void > const&);
+yes_type check(boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void > const&);
+#if !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
+yes_type check(std::string_view const&);
+yes_type check(std::wstring_view const&);
+#endif
+yes_type check(boost::basic_string_view< char, std::char_traits< char > > const&);
+yes_type check(boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const&);
+#if !defined(BOOST_NO_CXX11_NULLPTR)
+no_type check(std::nullptr_t);
+#endif
+no_type check(...);
+
+} // namespace is_convertible_to_path_source_impl
 
 //! The type trait indicates whether the type has a conversion path to one of the path source types
 template< typename T >
-struct is_convertible_to_path_source
+struct is_convertible_to_path_source :
+    public boost::integral_constant<
+        bool,
+        sizeof(is_convertible_to_path_source_impl::check(boost::declval< T const& >())) == sizeof(yes_type)
+    >
 {
-    // Note: The obscure naming of this function is a workaround for an MSVC-9.0 bug, which looks up the function outside the class scope
-    static yes_type _check_convertible_to_path_source(const char*);
-    static yes_type _check_convertible_to_path_source(const wchar_t*);
-    static yes_type _check_convertible_to_path_source(std::string const&);
-    static yes_type _check_convertible_to_path_source(std::wstring const&);
-    static yes_type _check_convertible_to_path_source(boost::container::basic_string< char, std::char_traits< char >, void > const&);
-    static yes_type _check_convertible_to_path_source(boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void > const&);
-#if !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
-    static yes_type _check_convertible_to_path_source(std::string_view const&);
-    static yes_type _check_convertible_to_path_source(std::wstring_view const&);
-#endif
-    static yes_type _check_convertible_to_path_source(boost::basic_string_view< char, std::char_traits< char > > const&);
-    static yes_type _check_convertible_to_path_source(boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const&);
-    static no_type _check_convertible_to_path_source(...);
-
-    static BOOST_CONSTEXPR_OR_CONST bool value =
-        sizeof(is_convertible_to_path_source< T >::_check_convertible_to_path_source(boost::declval< T const& >())) == sizeof(yes_type);
 };
 
-#else // !defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+#else // !defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 
 // Note: We use separate checks for convertibility to std::string_view and other types to avoid ambiguity with an implicit range constructor
 //       of std::string_view in the early C++23 draft (N4892). If a user's type is convertible to e.g. std::string and also satisfies
@@ -503,33 +521,50 @@ struct is_convertible_to_path_source
 //       through the conversion operator in the user's class and is also convertible to std::string_view through the implicit conversion
 //       constructor in std::string_view. The solution is to check convertibility to std::string_view separately first.
 
-template< typename T >
-struct is_convertible_to_std_string_view
-{
-    static yes_type _check_convertible_to_std_string_view(std::string_view const&);
-    static yes_type _check_convertible_to_std_string_view(std::wstring_view const&);
-    static no_type _check_convertible_to_std_string_view(...);
+namespace is_convertible_to_std_string_view_impl {
 
-    static BOOST_CONSTEXPR_OR_CONST bool value =
-        sizeof(is_convertible_to_std_string_view< T >::_check_convertible_to_std_string_view(boost::declval< T const& >())) == sizeof(yes_type);
+yes_type check(std::string_view const&);
+yes_type check(std::wstring_view const&);
+#if !defined(BOOST_NO_CXX11_NULLPTR)
+no_type check(std::nullptr_t);
+#endif
+no_type check(...);
+
+} // namespace is_convertible_to_std_string_view_impl
+
+template< typename T >
+struct is_convertible_to_std_string_view :
+    public boost::integral_constant<
+        bool,
+        sizeof(is_convertible_to_std_string_view_impl::check(boost::declval< T const& >())) == sizeof(yes_type)
+    >
+{
 };
 
-template< typename T >
-struct is_convertible_to_path_source_non_std_string_view
-{
-    // Note: The obscure naming of this function is a workaround for an MSVC-9.0 bug, which looks up the function outside the class scope
-    static yes_type _check_convertible_to_path_source(const char*);
-    static yes_type _check_convertible_to_path_source(const wchar_t*);
-    static yes_type _check_convertible_to_path_source(std::string const&);
-    static yes_type _check_convertible_to_path_source(std::wstring const&);
-    static yes_type _check_convertible_to_path_source(boost::container::basic_string< char, std::char_traits< char >, void > const&);
-    static yes_type _check_convertible_to_path_source(boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void > const&);
-    static yes_type _check_convertible_to_path_source(boost::basic_string_view< char, std::char_traits< char > > const&);
-    static yes_type _check_convertible_to_path_source(boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const&);
-    static no_type _check_convertible_to_path_source(...);
+namespace is_convertible_to_path_source_non_std_string_view_impl {
 
-    static BOOST_CONSTEXPR_OR_CONST bool value =
-        sizeof(is_convertible_to_path_source_non_std_string_view< T >::_check_convertible_to_path_source(boost::declval< T const& >())) == sizeof(yes_type);
+yes_type check(const char*);
+yes_type check(const wchar_t*);
+yes_type check(std::string const&);
+yes_type check(std::wstring const&);
+yes_type check(boost::container::basic_string< char, std::char_traits< char >, void > const&);
+yes_type check(boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void > const&);
+yes_type check(boost::basic_string_view< char, std::char_traits< char > > const&);
+yes_type check(boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const&);
+#if !defined(BOOST_NO_CXX11_NULLPTR)
+no_type check(std::nullptr_t);
+#endif
+no_type check(...);
+
+} // namespace is_convertible_to_path_source_non_std_string_view_impl
+
+template< typename T >
+struct is_convertible_to_path_source_non_std_string_view :
+    public boost::integral_constant<
+        bool,
+        sizeof(is_convertible_to_path_source_non_std_string_view_impl::check(boost::declval< T const& >())) == sizeof(yes_type)
+    >
+{
 };
 
 //! The type trait indicates whether the type has a conversion path to one of the path source types
@@ -538,11 +573,11 @@ struct is_convertible_to_path_source :
     public boost::disjunction<
         is_convertible_to_std_string_view< T >,
         is_convertible_to_path_source_non_std_string_view< T >
-    >
+    >::type
 {
 };
 
-#endif // !defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+#endif // !defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 
 //! The type trait makes \a T dependent on the second template argument. Used to delay type resolution and name binding.
 template< typename T, typename >
@@ -552,35 +587,35 @@ struct make_dependent
 };
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl(const char* source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(const char* source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< const char*, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl(const wchar_t* source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(const wchar_t* source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< const wchar_t*, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl(std::string const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(std::string const& source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< std::string, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl(std::wstring const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(std::wstring const& source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< std::wstring, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl
 (
     boost::container::basic_string< char, std::char_traits< char >, void > const& source,
     Callback cb,
@@ -588,11 +623,11 @@ BOOST_FORCEINLINE void dispatch_convertible_impl
 )
 {
     typedef typename path_traits::make_dependent< boost::container::basic_string< char, std::char_traits< char >, void >, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl
 (
     boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void > const& source,
     Callback cb,
@@ -600,11 +635,11 @@ BOOST_FORCEINLINE void dispatch_convertible_impl
 )
 {
     typedef typename path_traits::make_dependent< boost::container::basic_string< wchar_t, std::char_traits< wchar_t >, void >, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl
 (
     boost::basic_string_view< char, std::char_traits< char > > const& source,
     Callback cb,
@@ -612,11 +647,11 @@ BOOST_FORCEINLINE void dispatch_convertible_impl
 )
 {
     typedef typename path_traits::make_dependent< boost::basic_string_view< char, std::char_traits< char > >, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl
 (
     boost::basic_string_view< wchar_t, std::char_traits< wchar_t > > const& source,
     Callback cb,
@@ -624,71 +659,73 @@ BOOST_FORCEINLINE void dispatch_convertible_impl
 )
 {
     typedef typename path_traits::make_dependent< boost::basic_string_view< wchar_t, std::char_traits< wchar_t > >, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
-#if !defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+#if !defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 
 #if !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl(std::string_view const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(std::string_view const& source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< std::string_view, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_impl(std::wstring_view const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_impl(std::wstring_view const& source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< std::wstring_view, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 #endif // !defined(BOOST_NO_CXX17_HDR_STRING_VIEW)
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
 {
     typedef typename boost::remove_cv< Source >::type source_t;
-    path_traits::dispatch_convertible_impl< source_t >(source, cb, cvt);
+    return path_traits::dispatch_convertible_impl< source_t >(source, cb, cvt);
 }
 
-#else // !defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+#else // !defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_sv_impl(std::string_view const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_sv_impl(std::string_view const& source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< std::string_view, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
-BOOST_FORCEINLINE void dispatch_convertible_sv_impl(std::wstring_view const& source, Callback cb, const codecvt_type* cvt)
+BOOST_FORCEINLINE typename Callback::result_type dispatch_convertible_sv_impl(std::wstring_view const& source, Callback cb, const codecvt_type* cvt)
 {
     typedef typename path_traits::make_dependent< std::wstring_view, Source >::type source_t;
-    path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
+    return path_traits::dispatch(static_cast< source_t const& >(source), cb, cvt);
 }
 
 template< typename Source, typename Callback >
 BOOST_FORCEINLINE typename boost::disable_if_c<
-    is_convertible_to_std_string_view< typename boost::remove_cv< Source >::type >::value
+    is_convertible_to_std_string_view< typename boost::remove_cv< Source >::type >::value,
+    typename Callback::result_type
 >::type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
 {
     typedef typename boost::remove_cv< Source >::type source_t;
-    path_traits::dispatch_convertible_impl< source_t >(source, cb, cvt);
+    return path_traits::dispatch_convertible_impl< source_t >(source, cb, cvt);
 }
 
 template< typename Source, typename Callback >
 BOOST_FORCEINLINE typename boost::enable_if_c<
-    is_convertible_to_std_string_view< typename boost::remove_cv< Source >::type >::value
+    is_convertible_to_std_string_view< typename boost::remove_cv< Source >::type >::value,
+    typename Callback::result_type
 >::type dispatch_convertible(Source const& source, Callback cb, const codecvt_type* cvt = NULL)
 {
     typedef typename boost::remove_cv< Source >::type source_t;
-    path_traits::dispatch_convertible_sv_impl< source_t >(source, cb, cvt);
+    return path_traits::dispatch_convertible_sv_impl< source_t >(source, cb, cvt);
 }
 
-#endif // !defined(BOOST_FILESYSTEM_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
+#endif // !defined(BOOST_FILESYSTEM_DETAIL_CXX23_STRING_VIEW_HAS_IMPLICIT_RANGE_CTOR)
 
 } // namespace path_traits
 } // namespace detail
