@@ -152,34 +152,33 @@ void test_areal()
         TEST_DIFFERENCE_WITH(0, 1, ggl_list_20120221_volker, 2, 7962.66, 2, 2775258.93, 4);
     }
 
+#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
     {
-        // With rescaling, A is invalid (this is a robustness problem) and the other
-        // output is discarded because of zero area
-        // POSTGIS areas: 3.75893745345145, 2.5810000723917e-15
+        // 1: Very small sliver for B (discarded when rescaling)
+        // 2: sym difference is not considered as valid (without rescaling
+        //    this is a false negative)
+        // 3: with rescaling A is considered as invalid (robustness problem)
         ut_settings settings;
-        settings.sym_difference = BG_IF_RESCALED(false, true);
-        settings.set_test_validity(BG_IF_RESCALED(false, true));
-#if defined(BOOST_GEOMETRY_USE_RESCALING) || ! defined(BOOST_GEOMETRY_USE_KRAMER_RULE)
-        // No output for B
-        TEST_DIFFERENCE_WITH(0, 1, bug_21155501, 1, 3.758937, 0, 0.0, 1);
-#else
-        // Very small sliver for B, and sym difference is not considered valid
-        settings.set_test_validity(false);
-        TEST_DIFFERENCE_WITH(0, 1, bug_21155501, 1, 3.758937, 1, 1.7763568394002505e-15, 2);
-#endif
+        settings.validity_of_sym = BG_IF_RESCALED(false, true);
+        settings.validity_false_negative_sym = true;
+        TEST_DIFFERENCE_WITH(0, 1, bug_21155501,
+                             (count_set(1, 4)), expectation_limits(3.75893, 3.75894),
+                             (count_set(1, 4)), (expectation_limits(1.776357e-15, 7.661281e-15)),
+                             (count_set(2, 5)));
     }
+#endif
 
 #if defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
     {
         // With rescaling, it is complete but invalid
         // Without rescaling, one ring is missing (for a and s)
         ut_settings settings;
-        settings.percentage = 0.001;
         settings.set_test_validity(BG_IF_RESCALED(false, true));
+        settings.validity_of_sym = BG_IF_RESCALED(false, true);
         TEST_DIFFERENCE_WITH(0, 1, ticket_9081,
                              2, 0.0907392476356186,
                              4, 0.126018011439877,
-                             BG_IF_RESCALED(4, 3));
+                             count_set(3, 4));
     }
 #endif
 
@@ -191,20 +190,38 @@ void test_areal()
         settings.percentage = 0.001;
 
 #if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-        TEST_DIFFERENCE_WITH(0, 1, issue_630_a, 0, 0.0, 1, BG_IF_KRAMER(2.023326, 2.200326), 1);
+        TEST_DIFFERENCE_WITH(0, 1, issue_630_a, 0, expectation_limits(0.0), 1, (expectation_limits(2.023, 2.2004)), 1);
 #endif
-        TEST_DIFFERENCE_WITH(0, 1, issue_630_b, 1, 0.0056089, 2, 1.498976, 3);
-#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
 
-#if defined(BOOST_GEOMETRY_USE_KRAMER) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-        // Only succeeds with Kramer rule and no rescaling
+        TEST_DIFFERENCE_WITH(0, 1, issue_630_b, 1, 0.0056089, 2, 1.498976, 3);
+
+#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
         TEST_DIFFERENCE_WITH(0, 1, issue_630_c, 0, 0, 1, 1.493367, 1);
 #endif
 
-        TEST_DIFFERENCE_WITH(0, 1, issue_643, 1, 76.5385, BG_IF_KRAMER(1, 0), BG_IF_KRAMER(2.8634e-09, 0.0), 1);
+#if ! defined(BOOST_GEOMETRY_USE_RESCALING) || defined(BOOST_GEOMETRY_TEST_FAILURES)
+        // Symmetrical difference fails without get_clusters
+        settings.sym_difference = BG_IF_TEST_FAILURES;
+        TEST_DIFFERENCE_WITH(0, 1, issue_643, 1, expectation_limits(76.5385), optional(), optional_sliver(1.0e-6), 1);
 #endif
     }
 
+    // Cases below go (or went) wrong in either a ( [0] - [1] ) or b ( [1] - [0] )
+    // Requires reversal of isolation in ii turns. There should be 3 rings.
+    TEST_DIFFERENCE(issue_869_a, 3, 3600, 0, 0, 3); // a went wrong
+
+    TEST_DIFFERENCE(issue_888_34, 22, 0.2506824, 6, 0.0253798, 28); // a went wrong
+    TEST_DIFFERENCE(issue_888_37, 15, 0.0451408, 65, 0.3014843, 80); // b went wrong
+
+    {
+        ut_settings settings;
+        settings.validity_false_negative_a = true;
+        settings.validity_false_negative_sym = true;
+        TEST_DIFFERENCE_WITH(0, 1, issue_888_53, 117, 0.2973268, 17, 0.0525798, 134);
+    }
+
+    TEST_DIFFERENCE(issue_900, 0, 0.0, 2, 35, 2);
+
     // Areas and #clips correspond with POSTGIS (except sym case)
     test_one<Polygon, MultiPolygon, MultiPolygon>("case_101_multi",
         case_101_multi[0], case_101_multi[1],
@@ -342,11 +359,7 @@ void test_areal()
     TEST_DIFFERENCE(case_recursive_boxes_59, 8, 6.5, 7, 7.0, 12);
     TEST_DIFFERENCE(case_recursive_boxes_60, 6, 5.25, 7, 5.25, 11);
     TEST_DIFFERENCE(case_recursive_boxes_61, 2, 1.5, 6, 2.0, 7);
-#if defined(BOOST_GEOMETRY_TEST_FAILURES)
-    // Misses one triangle, should be fixed in traversal.
-    // It is not related to rescaling.
     TEST_DIFFERENCE(case_recursive_boxes_62, 5, 5.0, 11, 5.75, 12);
-#endif
 
     TEST_DIFFERENCE(case_recursive_boxes_63, 9, 10.5, 5, 27.75, 4);
     TEST_DIFFERENCE(case_recursive_boxes_64, 6, 2.75, 7, 4.5, 11);
@@ -367,8 +380,8 @@ void test_areal()
     TEST_DIFFERENCE(case_recursive_boxes_79, 2, 1.25, 6, 4.5, 8);
 
     // one polygon is divided into two, for same reason as union creates a small
-    // interior ring there
-    TEST_DIFFERENCE(case_recursive_boxes_80, 1, 0.5, 2, 0.75, BG_IF_RESCALED(3, 2));
+    // interior ring there, which is acceptable
+    TEST_DIFFERENCE(case_recursive_boxes_80, 1, 0.5, 2, 0.75, count_set(2, 3));
 
     TEST_DIFFERENCE(case_recursive_boxes_81, 3, 5.0, 6, 6.75, 6);
     TEST_DIFFERENCE(case_recursive_boxes_82, 5, 7.25, 7, 4.5, 8);
@@ -383,26 +396,24 @@ void test_areal()
     TEST_DIFFERENCE(case_recursive_boxes_88, 3, 4.75, 5, 6.75, 4);
 
     // Output of A can be 0 or 1 polygons (with a very small area)
-    TEST_DIFFERENCE(case_precision_m1, optional(), 0.0, 1, 57.0, count_set(1, 2));
+    TEST_DIFFERENCE(case_precision_m1, optional(), expectation_limits(0.0, 5.0e-7), 1, 57.0, count_set(1, 2));
     // Output of A can be 1 or 2 polygons (one with a very small area)
     TEST_DIFFERENCE(case_precision_m2, count_set(1, 2), 1.0, 1, 57.75, count_set(2, 3));
 
     {
-        ut_settings sym_settings;
-    #if ! defined(BOOST_GEOMETRY_USE_RESCALING)
-        sym_settings.sym_difference = false;
-    #endif
+        ut_settings settings;
+        settings.sym_difference = BG_IF_RESCALED(true, BG_IF_TEST_FAILURES);
         test_one<Polygon, MultiPolygon, MultiPolygon>("mysql_21965285_b",
             mysql_21965285_b[0],
             mysql_21965285_b[1],
             2, -1, 183.71376870369406,
             2, -1, 131.21376870369406,
-            sym_settings);
+            settings);
     }
 
     TEST_DIFFERENCE(mysql_regression_1_65_2017_08_31,
-                    optional(), BG_IF_RESCALED(4.30697514e-7, 0),
-                    3, 152.0642, count_set(3, 4));
+                    optional(), optional_sliver(1e-6),
+                    3, 152.064185, count_set(3, 4));
 }
 
 
@@ -427,10 +438,10 @@ void test_specific_areal()
         settings.set_test_validity(false);
 
         TEST_DIFFERENCE_WITH(0, 1, ticket_11674,
-                             BG_IF_KRAMER(3, 4),
-                             BG_IF_KRAMER(9105781.5, 9105473.5),
-                             5,
-                             BG_IF_KRAMER(119059.5, 119423), ignore_count());
+                             count_set(2, 3, 4),
+                             expectation_limits(9105196, 9105705),
+                             6,
+                             expectation_limits(119059, 119757), ignore_count());
     }
 
     {
@@ -441,19 +452,12 @@ void test_specific_areal()
         settings.remove_spikes = true;
 
         TEST_DIFFERENCE_WITH(0, 1, ticket_12751, 1,
-                             BG_IF_KRAMER(2781965.0, 2782114), 1,
-                             BG_IF_KRAMER(597.0, 598.0), 2);
-
-#if ! defined(BOOST_GEOMETRY_USE_KRAMER)
-        // Fails with general line form intersection, symmetric version misses one outcut
-        // TODO GENERAL FORM
-        settings.set_test_validity(false);
-        settings.sym_difference = false;
-#endif
+                             expectation_limits(2781964, 2782115), 1,
+                             expectation_limits(597.0, 598.0), 2);
 
         TEST_DIFFERENCE_WITH(2, 3, ticket_12751,
-                             2, BG_IF_KRAMER(2537992.5, 2538305),
-                             2, BG_IF_KRAMER(294963.5, 294737),
+                             2, expectation_limits(2537404, 2538306),
+                             2, expectation_limits(294736, 295353),
                              3);
     }
 
@@ -463,42 +467,38 @@ void test_specific_areal()
         ut_settings settings;
         settings.remove_spikes = true;
         settings.sym_difference = false;
+        settings.set_test_validity(false);
         TEST_DIFFERENCE_WITH(0, 1, ticket_12752,
-                             BG_IF_KRAMER(3, 2), BG_IF_KRAMER(2776692.0, 2776657),
-                             3, BG_IF_KRAMER(7893.0, 7710.5),
+                             count_set(2, 3), expectation_limits(2775740, 2776693),
+                             3, expectation_limits(7710, 7903),
                              2);
     }
 
     {
-#if defined(BOOST_GEOMETRY_USE_KRAMER) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-        // Fails completely with general line form intersection
-        // There is something with scale.
-        // TODO GENERAL FORM
         const std::string a_min_b =
-            TEST_DIFFERENCE(ticket_10661, 2, 1441632.5, 2, 13167454, 4);
+            test_one<Polygon, MultiPolygon, MultiPolygon>("ticket_10661",
+                          ticket_10661[0], ticket_10661[1],
+                          2, -1, expectation_limits(1441855, 1442081),
+                          2, -1, expectation_limits(13167454, 13182415),
+                          count_set(3, 4));
 
         test_one<Polygon, MultiPolygon, MultiPolygon>("ticket_10661_2",
             a_min_b, ticket_10661[2],
-            1, 8, 825192.0,
-            1, 10, BG_IF_KRAMER(27226370.5, 27842811),
-            1, -1, 825192.0 + 27226370.5);
-#endif
+            1, 8, 825640.5,
+            1, 10, expectation_limits(27226148, 27842812),
+            count_set(1, 2));
     }
 
     {
         ut_settings settings;
         settings.sym_difference = false;
 
-#if defined(BOOST_GEOMETRY_USE_KRAMER) || defined(BOOST_GEOMETRY_TEST_FAILURES)
-        // Fails with general line form intersection
-        // Misses one clip
-        // TODO GENERAL FORM
-        TEST_DIFFERENCE_WITH(0, 1, ticket_9942, 4, 7427727.5, 4,
-                             BG_IF_KRAMER(131506, 130083.5), 4);
-#endif
-        TEST_DIFFERENCE_WITH(0, 1, ticket_9942a, 2,
-                             BG_IF_KRAMER(412676.5, 413183.5), 2,
-                             BG_IF_KRAMER(76779.5, 76924), 4);
+        TEST_DIFFERENCE_WITH(0, 1, ticket_9942, 4,
+                             expectation_limits(7427727, 7428549), 5,
+                             expectation_limits(184653, 184687), 5);
+        TEST_DIFFERENCE_WITH(0, 1, ticket_9942a, 1,
+                             expectation_limits(412676, 413469), 2,
+                             expectation_limits(76779, 77038), 4);
     }
 }
 
@@ -520,16 +520,12 @@ int test_main(int, char* [])
 
 #if ! defined(BOOST_GEOMETRY_TEST_ONLY_ONE_TYPE)
     test_all<bg::model::d2::point_xy<float> >();
-
-#if defined(HAVE_TTMATH)
-    std::cout << "Testing TTMATH" << std::endl;
-    test_all<bg::model::d2::point_xy<ttmath_big> >();
-#endif
-
 #endif
 
 #if defined(BOOST_GEOMETRY_TEST_FAILURES)
-    BoostGeometryWriteExpectedFailures(23, 13);
+    // Not yet fully tested for float.
+    // The difference algorithm can generate (additional) slivers
+    BoostGeometryWriteExpectedFailures(24, 11, 21, 7);
 #endif
 
     return 0;
