@@ -25,7 +25,30 @@
 #include "test_suite.hpp"
 #include "checking_resource.hpp"
 
-BOOST_JSON_NS_BEGIN
+namespace boost {
+namespace json {
+
+namespace {
+
+struct throws_on_convert
+{
+    // this member only exists due to MSVC code analysis bug that marks lines
+    // in callers of the type's operator key_value_pair() as unreachable (due
+    // to exception thrown), even if that caller is a function template, and
+    // the line is reachable in other instantiations
+    bool should_throw = true;
+
+    throws_on_convert() = default;
+
+    operator key_value_pair()
+    {
+        if( should_throw )
+            throw std::invalid_argument("");
+        return key_value_pair( "", nullptr);
+    }
+};
+
+} // namespace
 
 BOOST_STATIC_ASSERT( std::is_nothrow_destructible<object>::value );
 BOOST_STATIC_ASSERT( std::is_nothrow_move_constructible<object>::value );
@@ -1310,24 +1333,21 @@ public:
         {
             BOOST_TEST(
                 o1.at("a").is_number());
-            BOOST_TEST_THROWS((o1.at("d")),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( (o1.at("d")) );
         }
 
         // at(key) const&
         {
             BOOST_TEST(
                 co1.at("a").is_number());
-            BOOST_TEST_THROWS((co1.at("d")),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( (co1.at("d")) );
         }
 
         // at(key) &&
         {
             BOOST_TEST(
                 std::move(o1).at("a").is_number());
-            BOOST_TEST_THROWS((std::move(o1).at("d")),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( (std::move(o1).at("d")) );
             value&& rv = std::move(o1).at("a");
             (void)rv;
         }
@@ -1597,6 +1617,46 @@ public:
             object({{"b",2}, {"c",3}})));
     }
 
+    void
+    testStrongGurantee()
+    {
+        // We used to preemptively reserve storage even if we don't add a new
+        // element. That violated strong guarantee requirement. This test
+        // checks we don't do that any more.
+
+        object o;
+        o.reserve(100);
+        std::size_t const capacity = o.capacity();
+        for( std::size_t i = 0; i < o.capacity() ; ++i )
+            o.emplace( std::to_string(i), i );
+        BOOST_ASSERT( capacity == o.capacity() );
+
+        BOOST_TEST( !o.emplace("0", 0).second );
+        BOOST_TEST( capacity == o.capacity() );
+
+        BOOST_TEST( !o.insert_or_assign("0", 0).second );
+        BOOST_TEST( capacity == o.capacity() );
+
+        o["0"] = 0;
+        BOOST_TEST( capacity == o.capacity() );
+
+        o.insert( key_value_pair("0", nullptr) );
+        BOOST_TEST( capacity == o.capacity() );
+
+        // Check that insertion rolls back reserve when cannot insert all
+        // elements.
+        std::array<throws_on_convert, 10> input;
+        try
+        {
+            o.insert( input.begin(), input.end() );
+        }
+        catch( ... )
+        {
+            // ignore
+        }
+        BOOST_TEST( capacity == o.capacity() );
+    }
+
     void
     run()
     {
@@ -1612,9 +1672,11 @@ public:
         testEquality();
         testAllocation();
         testHash();
+        testStrongGurantee();
     }
 };
 
 TEST_SUITE(object_test, "boost.json.object");
 
-BOOST_JSON_NS_END
+} // namespace json
+} // namespace boost
