@@ -22,7 +22,8 @@
 #include "test.hpp"
 #include "test_suite.hpp"
 
-BOOST_JSON_NS_BEGIN
+namespace boost {
+namespace json {
 
 BOOST_STATIC_ASSERT( std::is_nothrow_destructible<string>::value );
 BOOST_STATIC_ASSERT( std::is_nothrow_move_constructible<string>::value );
@@ -1037,8 +1038,7 @@ public:
             s2.at(1) = 'B';
             BOOST_TEST(s2.at(1) == 'B');
 
-            BOOST_TEST_THROWS(s1.at(s2.size()),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( s1.at(s2.size()) );
         }
 
         // at(size_type) const
@@ -1046,8 +1046,7 @@ public:
             BOOST_TEST(cs1.at(1) == 'b');
             BOOST_TEST(cs2.at(1) == 'B');
 
-            BOOST_TEST_THROWS(cs1.at(cs2.size()),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( cs1.at(cs2.size()) );
         }
 
         // operator[&](size_type)
@@ -1390,17 +1389,15 @@ public:
             // pos out of range
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(s.size() + 2, 1, '*')),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.insert(s.size() + 2, 1, '*')) );
             }
 
             // size > max_size
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(1, s.max_size(), 'a')),
-                    std::length_error);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    s.insert(1, s.max_size(), 'a') );
             }
         }
 
@@ -1425,9 +1422,8 @@ public:
             // pos out of range
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(s.size() + 2, "*")),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.insert(s.size() + 2, "*")));
             }
         }
 
@@ -1647,9 +1643,8 @@ public:
             // pos out of range
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(s.size() + 2, "*")),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.insert(s.size() + 2, "*")));
             }
         }
 
@@ -1756,9 +1751,8 @@ public:
 
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.erase(t.v1.size() + 1, 1)),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.erase(t.v1.size() + 1, 1)));
             }
         }
 
@@ -2174,8 +2168,8 @@ public:
             fail_loop([&](storage_ptr const& sp)
             {
                 string s(t.v2, sp);
-                BOOST_TEST_THROWS(s.replace(s.size() + 1, 1, t.v2),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    s.replace(s.size() + 1, 1, t.v2));
             });
 
             // outside, shrink
@@ -2397,8 +2391,8 @@ public:
             fail_loop([&](storage_ptr const& sp)
             {
                 string s(t.v2, sp);
-                BOOST_TEST_THROWS(s.replace(s.size() + 1, 1, 1, 'a'),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    s.replace(s.size() + 1, 1, 1, 'a'));
             });
         }
 
@@ -2770,13 +2764,13 @@ public:
         {
             std::unordered_set<string>(
                 0,
-                std::hash<string>(32));
+                std::hash<string>());
         }
 #endif
         {
-            std::hash<string> h1(32);
+            std::hash<string> h1;
             std::hash<string> h2(h1);
-            std::hash<string> h3(59);
+            std::hash<string> h3;
             h1 = h3;
             h2 = h3;
             (void)h2;
@@ -2823,4 +2817,5 @@ public:
 
 TEST_SUITE(string_test, "boost.json.string");
 
-BOOST_JSON_NS_END
+} // namespace json
+} // namespace boost
