@@ -9,11 +9,20 @@
 #include <cmath>
 #include <type_traits>
 #include <boost/math/tools/is_constant_evaluated.hpp>
+#include <boost/math/special_functions/fpclassify.hpp>
+
+#include <boost/math/tools/is_standalone.hpp>
+#ifndef BOOST_MATH_STANDALONE
+#include <boost/config.hpp>
+#ifdef BOOST_NO_CXX17_IF_CONSTEXPR
+#error "The header <boost/math/norms.hpp> can only be used in C++17 and later."
+#endif
+#endif
 
 namespace boost::math::ccmath {
 
 template <typename T>
-inline constexpr bool isnan(T x)
+inline constexpr bool isnan BOOST_PREVENT_MACRO_SUBSTITUTION(T x)
 {
     if(BOOST_MATH_IS_CONSTANT_EVALUATED(x))
     {
@@ -21,15 +30,15 @@ inline constexpr bool isnan(T x)
     }
     else
     {
-        using std::isnan;
+        using boost::math::isnan;
 
         if constexpr (!std::is_integral_v<T>)
         {
-            return isnan(x);
+            return (isnan)(x);
         }
         else
         {
-            return isnan(static_cast<double>(x));
+            return (isnan)(static_cast<double>(x));
         }
     }
 }
