@@ -23,11 +23,16 @@
 #include <boost/math/special_functions/gamma.hpp>
 #include <boost/math/special_functions/sinc.hpp>
 #include <boost/type_traits/is_class.hpp>
+#include <boost/type_index.hpp>
 
 #ifdef BOOST_HAS_FLOAT128
 #include <boost/multiprecision/complex128.hpp>
 #endif
 
+#if __has_include(<stdfloat>)
+#  include <stdfloat>
+#endif
+
 using std::exp;
 using std::cos;
 using std::tan;
@@ -184,7 +189,7 @@ void test_right_limit_infinite()
     Real Q_expected;
     Real error;
     Real L1;
-    auto integrator = get_integrator<Real>();
+    const auto& integrator = get_integrator<Real>();
 
     // Example 12
     const auto f2 = [](const Real& t)->Real { return exp(-t)/sqrt(t); };
@@ -192,8 +197,8 @@ void test_right_limit_infinite()
     Q_expected = root_pi<Real>();
     Real tol_mult = 1;
     // Multiprecision type have higher error rates, probably evaluation of f() is less accurate:
-    if (std::numeric_limits<Real>::digits10 > std::numeric_limits<long double>::digits10)
-       tol_mult = 12;
+    if (!std::numeric_limits<Real>::digits10 || (std::numeric_limits<Real>::digits10 > 25))
+       tol_mult = 1200;
     else if (std::numeric_limits<Real>::digits10 > std::numeric_limits<double>::digits10)
        tol_mult = 5;
     BOOST_CHECK_CLOSE_FRACTION(Q, Q_expected, tol * tol_mult);
@@ -241,7 +246,7 @@ void test_left_limit_infinite()
     Real Q_expected;
     Real error;
     Real L1;
-    auto integrator = get_integrator<Real>();
+    const auto& integrator = get_integrator<Real>();
 
     // Example 11:
     #ifdef BOOST_MATH_STANDALONE
@@ -281,7 +286,7 @@ void test_nr_examples()
     Real Q_expected;
     Real L1;
     Real error;
-    auto integrator = get_integrator<Real>();
+    const auto& integrator = get_integrator<Real>();
 
     auto f0 = [] (Real)->Real { return (Real) 0; };
     Q = integrator.integrate(f0, get_convergence_tolerance<Real>(), &error, &L1);
@@ -322,10 +327,8 @@ void test_nr_examples()
     Q = integrator.integrate(f2, get_convergence_tolerance<Real>(), &error, &L1);
     Q_expected = half<Real>()*boost::math::tgamma((Real) 5/ (Real) 14);
     tol_mul = 1;
-    if (std::numeric_limits<Real>::is_specialized == false)
-       tol_mul = 6;
-    else if (std::numeric_limits<Real>::digits10 > 40)
-       tol_mul = 100;
+    if ((std::numeric_limits<Real>::is_specialized == false) || (std::numeric_limits<Real>::digits10 > 40))
+       tol_mul = 500;
     else
        tol_mul = 3;
     BOOST_CHECK_CLOSE_FRACTION(Q, Q_expected, tol_mul * tol);
@@ -371,7 +374,7 @@ void test_crc()
     Real Q_expected;
     Real L1;
     Real error;
-    auto integrator = get_integrator<Real>();
+    const auto& integrator = get_integrator<Real>();
 
     auto f0 = [](const Real& x)->Real { return x > boost::math::tools::log_max_value<Real>() ? Real(0) : Real(log(x)*exp(-x)); };
     Q = integrator.integrate(f0, get_convergence_tolerance<Real>(), &error, &L1);
@@ -425,7 +428,7 @@ void test_crc()
        Q = integrator.integrate(f3, get_convergence_tolerance<Real>(), &error, &L1);
        Q_expected = s/(a*a+s*s);
        if (std::numeric_limits<Real>::digits10 > std::numeric_limits<double>::digits10)
-          tol_mult = 5000; // we should really investigate this more??
+          tol_mult = 500000; // we should really investigate this more??
        BOOST_CHECK_CLOSE_FRACTION(Q, Q_expected, tol_mult*tol);
     }
 
@@ -448,8 +451,8 @@ void test_crc()
        Q_expected = 1 / sqrt(1 + s*s);
        tol_mult = 3;
        // Multiprecision type have higher error rates, probably evaluation of f() is less accurate:
-       if (std::numeric_limits<Real>::digits10 > std::numeric_limits<long double>::digits10)
-          tol_mult = 750;
+       if ((std::numeric_limits<Real>::digits10 > std::numeric_limits<long double>::digits10) || (std::numeric_limits<Real>::digits > 100) || !std::numeric_limits<Real>::digits)
+          tol_mult = 50000;
        BOOST_CHECK_CLOSE_FRACTION(Q, Q_expected, tol_mult * tol);
     }
     auto f6 = [](const Real& t)->Real { return t > boost::math::tools::log_max_value<Real>() ? Real(0) : Real(exp(-t*t)*log(t));};
@@ -502,7 +505,7 @@ void test_complex_modified_bessel()
     Real tol = 100 * boost::math::tools::epsilon<Real>();
     Real error;
     Real L1;
-    auto integrator = get_integrator<Real>();
+    const auto& integrator = get_integrator<Real>();
 
     // Integral Representation of Modified Complex Bessel function:
     // https://en.wikipedia.org/wiki/Bessel_function#Modified_Bessel_functions
@@ -542,7 +545,7 @@ void test_complex_exponential_integral_E1(){
     Real tol = 100 * boost::math::tools::epsilon<Real>();
     Real error;
     Real L1;
-    auto integrator = get_integrator<Real>();
+    const auto& integrator = get_integrator<Real>();
 
     Complex z{1.5,0.5};
 
@@ -587,17 +590,35 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
    */
 
 #ifdef TEST1
+
+#ifdef __STDCPP_FLOAT32_T__
+    test_left_limit_infinite<std::float32_t>();
+    test_right_limit_infinite<std::float32_t>();
+    test_nr_examples<std::float32_t>();
+    test_crc<std::float32_t>();
+#else
     test_left_limit_infinite<float>();
     test_right_limit_infinite<float>();
     test_nr_examples<float>();
     test_crc<float>();
 #endif
+
+#endif
 #ifdef TEST2
+
+#ifdef __STDCPP_FLOAT64_T__
+    test_left_limit_infinite<std::float64_t>();
+    test_right_limit_infinite<std::float64_t>();
+    test_nr_examples<std::float64_t>();
+    test_crc<std::float64_t>();
+#else
     test_left_limit_infinite<double>();
     test_right_limit_infinite<double>();
     test_nr_examples<double>();
     test_crc<double>();
 #endif
+
+#endif
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
 #ifdef TEST3
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
@@ -608,7 +629,7 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
 #endif
 #endif
 #endif
-#if defined(TEST4) && !defined(BOOST_MATH_NO_MP_TESTS)
+#if defined(TEST4) && defined(BOOST_MATH_RUN_MP_TESTS)
     test_left_limit_infinite<cpp_bin_float_quad>();
     test_right_limit_infinite<cpp_bin_float_quad>();
     test_nr_examples<cpp_bin_float_quad>();
@@ -623,13 +644,13 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
     test_crc<boost::math::concepts::real_concept>();
 #endif
 #endif
-#if defined(TEST6) && !defined(BOOST_MATH_NO_MP_TESTS)
+#if defined(TEST6) && defined(BOOST_MATH_RUN_MP_TESTS)
     test_left_limit_infinite<boost::multiprecision::cpp_bin_float_50>();
     test_right_limit_infinite<boost::multiprecision::cpp_bin_float_50>();
     test_nr_examples<boost::multiprecision::cpp_bin_float_50>();
     test_crc<boost::multiprecision::cpp_bin_float_50>();
 #endif
-#if defined(TEST7) && !defined(BOOST_MATH_NO_MP_TESTS)
+#if defined(TEST7) && defined(BOOST_MATH_RUN_MP_TESTS)
     test_left_limit_infinite<boost::multiprecision::cpp_dec_float_50>();
     test_right_limit_infinite<boost::multiprecision::cpp_dec_float_50>();
     test_nr_examples<boost::multiprecision::cpp_dec_float_50>();
@@ -637,7 +658,7 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
     // This one causes stack overflows on the CI machine, but not locally,
     // assume it's due to restricted resources on the server, and <shrug> for now...
     //
-#if ! BOOST_WORKAROUND(BOOST_MSVC, == 1900) && !defined(BOOST_MATH_NO_MP_TESTS)
+#if ! BOOST_WORKAROUND(BOOST_MSVC, == 1900) && defined(BOOST_MATH_RUN_MP_TESTS)
     test_crc<boost::multiprecision::cpp_dec_float_50>();
 #endif
 #endif
@@ -657,7 +678,7 @@ BOOST_AUTO_TEST_CASE(exp_sinh_quadrature_test)
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
     test_complex_exponential_integral_E1<std::complex<long double>>();
 #endif
-#ifndef BOOST_MATH_NO_MP_TESTS
+#if defined(BOOST_MATH_RUN_MP_TESTS)
     test_complex_exponential_integral_E1<boost::multiprecision::cpp_complex_quad>();
 #endif
 #endif
