@@ -53,7 +53,7 @@ namespace test {
       {
         UNORDERED_SCOPE(object::object())
         {
-          UNORDERED_EPOINT("Mock object default constructor.");
+          UNORDERED_EPOINT("Mock object default constructor.")
         }
       }
 
@@ -61,7 +61,7 @@ namespace test {
       {
         UNORDERED_SCOPE(object::object(int))
         {
-          UNORDERED_EPOINT("Mock object constructor by value.");
+          UNORDERED_EPOINT("Mock object constructor by value.")
         }
       }
 
@@ -70,7 +70,7 @@ namespace test {
       {
         UNORDERED_SCOPE(object::object(object))
         {
-          UNORDERED_EPOINT("Mock object copy constructor.");
+          UNORDERED_EPOINT("Mock object copy constructor.")
         }
       }
 
@@ -85,7 +85,7 @@ namespace test {
         UNORDERED_SCOPE(object::operator=(object))
         {
           tag1_ = x.tag1_;
-          UNORDERED_EPOINT("Mock object assign operator 1.");
+          UNORDERED_EPOINT("Mock object assign operator 1.")
           tag2_ = x.tag2_;
           // UNORDERED_EPOINT("Mock object assign operator 2.");
         }
@@ -96,7 +96,7 @@ namespace test {
       {
         UNORDERED_SCOPE(operator==(object, object))
         {
-          UNORDERED_EPOINT("Mock object equality operator.");
+          UNORDERED_EPOINT("Mock object equality operator.")
         }
 
         return x1.tag1_ == x2.tag1_ && x1.tag2_ == x2.tag2_;
@@ -106,7 +106,7 @@ namespace test {
       {
         UNORDERED_SCOPE(operator!=(object, object))
         {
-          UNORDERED_EPOINT("Mock object inequality operator.");
+          UNORDERED_EPOINT("Mock object inequality operator.")
         }
 
         return !(x1.tag1_ == x2.tag1_ && x1.tag2_ == x2.tag2_);
@@ -150,7 +150,7 @@ namespace test {
       {
         UNORDERED_SCOPE(hash::object())
         {
-          UNORDERED_EPOINT("Mock hash default constructor.");
+          UNORDERED_EPOINT("Mock hash default constructor.")
         }
       }
 
@@ -158,7 +158,7 @@ namespace test {
       {
         UNORDERED_SCOPE(hash::hash(hash))
         {
-          UNORDERED_EPOINT("Mock hash copy constructor.");
+          UNORDERED_EPOINT("Mock hash copy constructor.")
         }
       }
 
@@ -166,9 +166,9 @@ namespace test {
       {
         UNORDERED_SCOPE(hash::operator=(hash))
         {
-          UNORDERED_EPOINT("Mock hash assign operator 1.");
+          UNORDERED_EPOINT("Mock hash assign operator 1.")
           tag_ = x.tag_;
-          UNORDERED_EPOINT("Mock hash assign operator 2.");
+          UNORDERED_EPOINT("Mock hash assign operator 2.")
         }
         return *this;
       }
@@ -177,7 +177,7 @@ namespace test {
       {
         UNORDERED_SCOPE(hash::operator()(object))
         {
-          UNORDERED_EPOINT("Mock hash function.");
+          UNORDERED_EPOINT("Mock hash function.")
         }
 
         return hash_impl(x);
@@ -187,7 +187,7 @@ namespace test {
       {
         UNORDERED_SCOPE(hash::operator()(std::pair<object, object>))
         {
-          UNORDERED_EPOINT("Mock hash pair function.");
+          UNORDERED_EPOINT("Mock hash pair function.")
         }
 
         return hash_impl(x.first) * 193ul + hash_impl(x.second) * 97ul + 29ul;
@@ -214,7 +214,7 @@ namespace test {
       {
         UNORDERED_SCOPE(operator==(hash, hash))
         {
-          UNORDERED_EPOINT("Mock hash equality function.");
+          UNORDERED_EPOINT("Mock hash equality function.")
         }
         return x1.tag_ == x2.tag_;
       }
@@ -223,12 +223,25 @@ namespace test {
       {
         UNORDERED_SCOPE(hash::operator!=(hash, hash))
         {
-          UNORDERED_EPOINT("Mock hash inequality function.");
+          UNORDERED_EPOINT("Mock hash inequality function.")
         }
         return x1.tag_ != x2.tag_;
       }
+
+#if defined(BOOST_UNORDERED_FOA_TESTS)
+      friend void swap(hash&, hash&) noexcept;
+#endif
     };
 
+#if defined(BOOST_UNORDERED_FOA_TESTS)
+      void swap(hash& lhs, hash& rhs) noexcept
+      {
+        int tag = lhs.tag_;
+        lhs.tag_ = rhs.tag_;
+        rhs.tag_ = tag;
+      }
+#endif
+
     class less
     {
       int tag_;
@@ -249,7 +262,7 @@ namespace test {
         if (less_impl(x1.first, x2.first)) {
           return true;
         }
-        if (!less_impl(x1.first, x2.first)) {
+        if (less_impl(x2.first, x1.first)) {
           return false;
         }
         return less_impl(x1.second, x2.second);
@@ -287,7 +300,7 @@ namespace test {
       {
         UNORDERED_SCOPE(equal_to::equal_to())
         {
-          UNORDERED_EPOINT("Mock equal_to default constructor.");
+          UNORDERED_EPOINT("Mock equal_to default constructor.")
         }
       }
 
@@ -295,7 +308,7 @@ namespace test {
       {
         UNORDERED_SCOPE(equal_to::equal_to(equal_to))
         {
-          UNORDERED_EPOINT("Mock equal_to copy constructor.");
+          UNORDERED_EPOINT("Mock equal_to copy constructor.")
         }
       }
 
@@ -303,9 +316,9 @@ namespace test {
       {
         UNORDERED_SCOPE(equal_to::operator=(equal_to))
         {
-          UNORDERED_EPOINT("Mock equal_to assign operator 1.");
+          UNORDERED_EPOINT("Mock equal_to assign operator 1.")
           tag_ = x.tag_;
-          UNORDERED_EPOINT("Mock equal_to assign operator 2.");
+          UNORDERED_EPOINT("Mock equal_to assign operator 2.")
         }
         return *this;
       }
@@ -314,7 +327,7 @@ namespace test {
       {
         UNORDERED_SCOPE(equal_to::operator()(object, object))
         {
-          UNORDERED_EPOINT("Mock equal_to function.");
+          UNORDERED_EPOINT("Mock equal_to function.")
         }
 
         return equal_impl(x1, x2);
@@ -326,7 +339,7 @@ namespace test {
         UNORDERED_SCOPE(equal_to::operator()(
           std::pair<object, object>, std::pair<object, object>))
         {
-          UNORDERED_EPOINT("Mock equal_to function.");
+          UNORDERED_EPOINT("Mock equal_to function.")
         }
 
         return equal_impl(x1.first, x2.first) &&
@@ -349,7 +362,7 @@ namespace test {
       {
         UNORDERED_SCOPE(operator==(equal_to, equal_to))
         {
-          UNORDERED_EPOINT("Mock equal_to equality function.");
+          UNORDERED_EPOINT("Mock equal_to equality function.")
         }
         return x1.tag_ == x2.tag_;
       }
@@ -358,14 +371,26 @@ namespace test {
       {
         UNORDERED_SCOPE(operator!=(equal_to, equal_to))
         {
-          UNORDERED_EPOINT("Mock equal_to inequality function.");
+          UNORDERED_EPOINT("Mock equal_to inequality function.")
         }
         return x1.tag_ != x2.tag_;
       }
 
       friend less create_compare(equal_to x) { return less(x.tag_); }
+#if defined(BOOST_UNORDERED_FOA_TESTS)
+      friend void swap(equal_to&, equal_to&) noexcept;
+#endif
     };
 
+#if defined(BOOST_UNORDERED_FOA_TESTS)
+    void swap(equal_to& lhs, equal_to& rhs) noexcept
+    {
+      int tag = lhs.tag_;
+      lhs.tag_ = rhs.tag_;
+      rhs.tag_ = tag;
+    }
+#endif
+
     template <class T> class allocator
     {
     public:
@@ -387,7 +412,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator::allocator())
         {
-          UNORDERED_EPOINT("Mock allocator default constructor.");
+          UNORDERED_EPOINT("Mock allocator default constructor.")
         }
         test::detail::tracker.allocator_ref();
       }
@@ -435,7 +460,7 @@ namespace test {
         T* ptr = 0;
         UNORDERED_SCOPE(allocator::allocate(size_type))
         {
-          UNORDERED_EPOINT("Mock allocator allocate function.");
+          UNORDERED_EPOINT("Mock allocator allocate function.")
 
           using namespace std;
           ptr = (T*)malloc(n * sizeof(T));
@@ -453,7 +478,7 @@ namespace test {
         T* ptr = 0;
         UNORDERED_SCOPE(allocator::allocate(size_type, const_pointer))
         {
-          UNORDERED_EPOINT("Mock allocator allocate function.");
+          UNORDERED_EPOINT("Mock allocator allocate function.")
 
           using namespace std;
           ptr = (T*)malloc(n * sizeof(T));
@@ -482,7 +507,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator::construct(U*, Arg))
         {
-          UNORDERED_EPOINT("Mock allocator construct function.");
+          UNORDERED_EPOINT("Mock allocator construct function.")
           new (p) U(t);
         }
         test::detail::tracker.track_construct((void*)p, sizeof(U), tag_);
@@ -492,7 +517,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator::construct(U*, BOOST_FWD_REF(Args)...))
         {
-          UNORDERED_EPOINT("Mock allocator construct function.");
+          UNORDERED_EPOINT("Mock allocator construct function.")
           new (p) U(boost::forward<Args>(args)...);
         }
         test::detail::tracker.track_construct((void*)p, sizeof(U), tag_);
@@ -510,7 +535,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator::construct(pointer, T))
         {
-          UNORDERED_EPOINT("Mock allocator max_size function.");
+          UNORDERED_EPOINT("Mock allocator max_size function.")
         }
         return (std::numeric_limits<std::size_t>::max)();
       }
@@ -567,7 +592,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator2::allocator2())
         {
-          UNORDERED_EPOINT("Mock allocator2 default constructor.");
+          UNORDERED_EPOINT("Mock allocator2 default constructor.")
         }
         test::detail::tracker.allocator_ref();
       }
@@ -616,7 +641,7 @@ namespace test {
         T* ptr = 0;
         UNORDERED_SCOPE(allocator2::allocate(size_type))
         {
-          UNORDERED_EPOINT("Mock allocator2 allocate function.");
+          UNORDERED_EPOINT("Mock allocator2 allocate function.")
 
           using namespace std;
           ptr = (T*)malloc(n * sizeof(T));
@@ -634,7 +659,7 @@ namespace test {
         T* ptr = 0;
         UNORDERED_SCOPE(allocator2::allocate(size_type, const_pointer))
         {
-          UNORDERED_EPOINT("Mock allocator2 allocate function.");
+          UNORDERED_EPOINT("Mock allocator2 allocate function.")
 
           using namespace std;
           ptr = (T*)malloc(n * sizeof(T));
@@ -663,7 +688,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator2::construct(U*, V))
         {
-          UNORDERED_EPOINT("Mock allocator2 construct function.");
+          UNORDERED_EPOINT("Mock allocator2 construct function.")
           new (p) U(v);
         }
         test::detail::tracker.track_construct((void*)p, sizeof(U), tag_);
@@ -674,7 +699,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator2::construct(U*, BOOST_FWD_REF(Args)...))
         {
-          UNORDERED_EPOINT("Mock allocator2 construct function.");
+          UNORDERED_EPOINT("Mock allocator2 construct function.")
           new (p) U(boost::forward<Args>(args)...);
         }
         test::detail::tracker.track_construct((void*)p, sizeof(U), tag_);
@@ -692,7 +717,7 @@ namespace test {
       {
         UNORDERED_SCOPE(allocator2::construct(pointer, T))
         {
-          UNORDERED_EPOINT("Mock allocator2 max_size function.");
+          UNORDERED_EPOINT("Mock allocator2 max_size function.")
         }
         return (std::numeric_limits<std::size_t>::max)();
       }
