@@ -11,7 +11,7 @@
 #define BOOST_URL_STRING_VIEW_HPP
 
 #include <boost/url/detail/config.hpp>
-#include <boost/core/detail/string_view.hpp>
+#include <boost/url/detail/string_view.hpp>
 
 namespace boost {
 namespace urls {
@@ -25,8 +25,26 @@ namespace urls {
     ensuring that the lifetime of character
     buffer extends until it is no longer
     referenced.
+
+    @note This alias is no longer supported and
+    should not be used in new code. Please use
+    `core::string_view` instead.
+
+    This alias is included for backwards
+    compatibility with earlier versions of the
+    library.
+
+    However, it will be removed in future releases,
+    and using it in new code is not recommended.
+
+    Please use the updated version instead to
+    ensure compatibility with future versions of
+    the library.
+
 */
-typedef boost::core::string_view string_view;
+using string_view
+    BOOST_URL_DEPRECATED("Use core::string_view instead") =
+    boost::core::string_view;
 
 } // urls
 } // boost
