@@ -12,6 +12,8 @@
 #include <boost/container/string.hpp>
 #include <boost/filesystem/path.hpp>
 #include <boost/core/ignore_unused.hpp>
+#include <boost/core/detail/string_view.hpp>
+#include <boost/url/string_view.hpp>
 
 //[snippet_headers_1
 #include <boost/url.hpp>
@@ -53,18 +55,18 @@ using_url_views()
     }
 
     //[code_urls_parsing_1
-    string_view s = "https://user:pass@example.com:443/path/to/my%2dfile.txt?id=42&name=John%20Doe+Jingleheimer%2DSchmidt#page%20anchor";
+    boost::core::string_view s = "https://user:pass@example.com:443/path/to/my%2dfile.txt?id=42&name=John%20Doe+Jingleheimer%2DSchmidt#page%20anchor";
     //]
 
     {
         //[code_urls_parsing_2
-        result<url_view> r = parse_uri( s );
+        boost::system::result<url_view> r = parse_uri( s );
         //]
         boost::ignore_unused(r);
     }
 
     {
-        result<url_view> r = parse_uri( s );
+        boost::system::result<url_view> r = parse_uri( s );
         //[snippet_parsing_3
         url_view u = r.value();
         //]
@@ -72,7 +74,7 @@ using_url_views()
     }
 
     {
-        result<url_view> r = parse_uri( s );
+        boost::system::result<url_view> r = parse_uri( s );
         //[snippet_parsing_4
         url_view u = *r;
         //]
@@ -194,7 +196,7 @@ using_url_views()
 #endif
     {
         //[snippet_decoding_5a
-        auto function = [](string_view str)
+        auto function = [](boost::core::string_view str)
         {
             std::cout << str << "\n";
         };
@@ -240,7 +242,7 @@ using_url_views()
 void
 using_urls()
 {
-    string_view s = "https://user:pass@www.example.com:443/path/to/my%2dfile.txt?id=42&name=John%20Doe#page%20anchor";
+    boost::core::string_view s = "https://user:pass@www.example.com:443/path/to/my%2dfile.txt?id=42&name=John%20Doe#page%20anchor";
 
     //[snippet_quicklook_modifying_1
     url u = parse_uri( s ).value();
@@ -277,8 +279,46 @@ void
 parsing_urls()
 {
     {
+        auto handle_error = [](boost::system::error_code e)
+        {
+            boost::ignore_unused(e);
+        };
         //[snippet_parsing_url_1
-        result< url > ru = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
+        boost::system::result< url > ru = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
+        if ( ru )
+        {
+            url u = *ru;
+            assert(u.encoded_path() == "/path/to/file.txt");
+        }
+        else
+        {
+            boost::system::error_code e = ru.error();
+            handle_error(e);
+        }
+        //]
+    }
+
+    {
+        auto handle_error = [](boost::system::system_error& e)
+        {
+            boost::ignore_unused(e);
+        };
+        //[snippet_parsing_url_1b
+        try
+        {
+            url u = parse_uri_reference( "https://www.example.com/path/to/file.txt" ).value();
+            assert(u.encoded_path() == "/path/to/file.txt");
+        }
+        catch (boost::system::system_error &e)
+        {
+            handle_error(e);
+        }
+        //]
+    }
+
+    {
+        //[snippet_parsing_url_1b2
+        boost::system::result< url > ru = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
         if ( ru.has_value() )
         {
             url u = *ru;
@@ -299,29 +339,29 @@ parsing_urls()
 
     {
         //[snippet_parsing_url_1bc
-        result< url > rv = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
+        boost::system::result< url > rv = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
 
-        static_assert( std::is_convertible< result< url_view >, result< url > >::value, "" );
+        static_assert( std::is_convertible< boost::system::result< url_view >, boost::system::result< url > >::value, "" );
         //]
         boost::ignore_unused(rv);
     }
 
     {
         //[snippet_parsing_url_1bd
-        result< static_url<1024> > rv = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
+        boost::system::result< static_url<1024> > rv = parse_uri_reference( "https://www.example.com/path/to/file.txt" );
 
-        static_assert( std::is_convertible< result< static_url<1024> >, result< url > >::value, "" );
+        static_assert( std::is_convertible< boost::system::result< static_url<1024> >, boost::system::result< url > >::value, "" );
         //]
         boost::ignore_unused(rv);
     }
 
     {
         //[snippet_parsing_url_1c
-        result< url_view > r0 = parse_relative_ref( "/path/to/file.txt" );
+        boost::system::result< url_view > r0 = parse_relative_ref( "/path/to/file.txt" );
         assert( r0.has_value() );
         //]
         //[snippet_parsing_url_1d
-        result< url_view > r1 = parse_uri( "https://www.example.com" );
+        boost::system::result< url_view > r1 = parse_uri( "https://www.example.com" );
         assert( r1.has_value() );
         url dest;
         resolve(r1.value(), r0.value(), dest);
@@ -417,6 +457,76 @@ parsing_components()
     }
 }
 
+void
+formatting_components()
+{
+    // I have spent a lot of time on this and have no
+    // idea how to fix this bug in GCC 4.8 and GCC 5.0
+    // without help from the pros.
+#if !BOOST_WORKAROUND( BOOST_GCC_VERSION, < 60000 )
+    {
+        //[snippet_format_1
+        url u = format("{}://{}:{}/rfc/{}", "https", "www.ietf.org", 80, "rfc2396.txt");
+        assert(u.buffer() == "https://www.ietf.org:80/rfc/rfc2396.txt");
+        //]
+    }
+
+    {
+        //[snippet_format_2
+        url u = format("https://{}/{}", "www.boost.org", "Hello world!");
+        assert(u.buffer() == "https://www.boost.org/Hello%20world!");
+        //]
+    }
+
+    {
+        //[snippet_format_3a
+        url u = format("{}:{}", "mailto", "someone@example.com");
+        assert(u.buffer() == "mailto:someone@example.com");
+        assert(u.scheme() == "mailto");
+        assert(u.path() == "someone@example.com");
+        //]
+    }
+    {
+        //[snippet_format_3b
+        url u = format("{}{}", "mailto:", "someone@example.com");
+        assert(u.buffer() == "mailto%3Asomeone@example.com");
+        assert(!u.has_scheme());
+        assert(u.path() == "mailto:someone@example.com");
+        assert(u.encoded_path() == "mailto%3Asomeone@example.com");
+        //]
+    }
+
+    {
+        //[snippet_format_4
+        static_url<50> u;
+        format_to(u, "{}://{}:{}/rfc/{}", "https", "www.ietf.org", 80, "rfc2396.txt");
+        assert(u.buffer() == "https://www.ietf.org:80/rfc/rfc2396.txt");
+        //]
+    }
+
+    {
+        //[snippet_format_5a
+        url u = format("{0}://{2}:{1}/{3}{4}{3}", "https", 80, "www.ietf.org", "abra", "cad");
+        assert(u.buffer() == "https://www.ietf.org:80/abracadabra");
+        //]
+    }
+    {
+        //[snippet_format_5b
+        url u = format("https://example.com/~{username}", arg("username", "mark"));
+        assert(u.buffer() == "https://example.com/~mark");
+        //]
+    }
+
+    {
+        //[snippet_format_5c
+        boost::core::string_view fmt = "{scheme}://{host}:{port}/{dir}/{file}";
+        url u = format(fmt, {{"scheme", "https"}, {"port", 80}, {"host", "example.com"}, {"dir", "path/to"}, {"file", "file.txt"}});
+        assert(u.buffer() == "https://example.com:80/path/to/file.txt");
+        //]
+    }
+#endif
+}
+
 void
 parsing_scheme()
 {
@@ -530,14 +640,14 @@ parsing_authority()
     }
     {
         struct resolve_f {
-            string_view
-            operator()(string_view h)
+            boost::core::string_view
+            operator()(boost::core::string_view h)
             {
                 return h;
             }
         } resolve;
         struct write_request_f {
-            void operator()(string_view) {}
+            void operator()(boost::core::string_view) {}
             void operator()(ipv4_address) {}
             void operator()(ipv6_address) {}
         } write_request;
@@ -704,7 +814,7 @@ parsing_path()
     {
         {
             //[snippet_parsing_path_5_a
-            string_view s = "https://www.boost.org";
+            boost::core::string_view s = "https://www.boost.org";
             url_view u(s);
             std::cout << u << "\n"
                       << "path:     " << u.encoded_host()            << "\n"
@@ -714,7 +824,7 @@ parsing_path()
         }
         {
             //[snippet_parsing_path_5_b
-            string_view s = "https://www.boost.org/";
+            boost::core::string_view s = "https://www.boost.org/";
             url_view u(s);
             std::cout << u << "\n"
                       << "host:     " << u.encoded_host()            << "\n"
@@ -724,7 +834,7 @@ parsing_path()
         }
         {
             //[snippet_parsing_path_5_c
-            string_view s = "https://www.boost.org//";
+            boost::core::string_view s = "https://www.boost.org//";
             url_view u(s);
             std::cout << u << "\n"
                       << "host:     " << u.encoded_host()            << "\n"
@@ -1087,7 +1197,7 @@ grammar_parse()
         //[snippet_parse_1
         // VFALCO we should not show this example
         /*
-        string_view s = "http:after_scheme";
+        boost::core::string_view s = "http:after_scheme";
         const char* it = s.begin();
         auto rv = grammar::parse(it, s.end(), scheme_rule() );
         if( ! rv )
@@ -1103,9 +1213,9 @@ grammar_parse()
         //[snippet_parse_2
         // VFALCO This needs refactoring
         /*
-        string_view s = "?key=value#anchor";
+        boost::core::string_view s = "?key=value#anchor";
         const char* it = s.begin();
-        error_code ec;
+        boost::system::error_code ec;
         if (grammar::parse(it, s.end(), ec, r1))
         {
             auto r2 = grammar::parse( it, s.end(), fragment_part_rule );
@@ -1123,10 +1233,10 @@ grammar_parse()
         //[snippet_parse_3
         // VFALCO This needs refactoring
         /*
-        string_view s = "?key=value#anchor";
+        boost::core::string_view s = "?key=value#anchor";
         query_part_rule r1;
         const char* it = s.begin();
-        error_code ec;
+        boost::system::error_code ec;
         auto r2 = grammar::parse( it, s.end(), ec, fragment_part_rule );
         if( ! ec.failed() )
         {
@@ -1140,9 +1250,9 @@ grammar_parse()
     {
         //[snippet_parse_4
         /* VFALCO This will be removed
-        string_view s = "http://www.boost.org";
+        boost::core::string_view s = "http://www.boost.org";
         uri_rule r;
-        error_code ec;
+        boost::system::error_code ec;
         if (grammar::parse_string(s, ec, r))
         {
             std::cout << "scheme: " << r.scheme_part.scheme << '\n';
@@ -1157,7 +1267,7 @@ grammar_parse()
 /* VFALCO This needs rewriting
 struct lowercase_rule
 {
-    string_view str;
+    boost::core::string_view str;
 
     friend
     void
@@ -1165,7 +1275,7 @@ struct lowercase_rule
         grammar::parse_tag const&,
         char const*& it,
         char const* const end,
-        error_code& ec,
+        boost::system::error_code& ec,
         lowercase_rule& t) noexcept
     {
         ec = {};
@@ -1174,7 +1284,7 @@ struct lowercase_rule
         {
             ++it;
         }
-        t.str = string_view(begin, it);
+        t.str = boost::core::string_view(begin, it);
     }
 };
 */
@@ -1187,10 +1297,10 @@ grammar_customization()
         //[snippet_customization_2
         // VFALCO THIS NEEDS TO BE PORTED
         /*
-        string_view s = "http:somelowercase";
+        boost::core::string_view s = "http:somelowercase";
         scheme_rule r1;
         lowercase_rule r2;
-        error_code ec;
+        boost::system::error_code ec;
         if (grammar::parse_string(s, ec, r1, ':', r2))
         {
             std::cout << "scheme: " << r1.scheme << '\n';
@@ -1486,7 +1596,7 @@ encoding()
 
     {
         //[snippet_encoding_4
-        string_view e = "hello world";
+        boost::core::string_view e = "hello world";
         std::string s;
         s.reserve(encoded_size(e, pchars));
         encode(e, pchars, {}, string_token::assign_to(s));
@@ -1496,7 +1606,7 @@ encoding()
 
     {
         //[snippet_encoding_5
-        string_view e = "hello world";
+        boost::core::string_view e = "hello world";
         std::string s;
         s.resize(encoded_size(e, pchars));
         encode(&s[0], s.size(), e, pchars);
@@ -1513,7 +1623,7 @@ encoding()
 
     {
         //[snippet_encoding_7
-        result<pct_string_view> rs =
+        boost::system::result<pct_string_view> rs =
             make_pct_string_view("hello%20world");
         assert(rs.has_value());
         pct_string_view sv = rs.value();
@@ -1559,7 +1669,7 @@ encoding()
 
     {
         //[snippet_encoding_12
-        result<pct_string_view> rs =
+        boost::system::result<pct_string_view> rs =
             make_pct_string_view("hello%20world");
         assert(rs.has_value());
         pct_string_view s = rs.value();
@@ -1585,7 +1695,7 @@ encoding()
         {
             pct_string_view seg0 = *it0;
             decode_view dseg0 = *seg0;
-            string_view seg1 = *it1;
+            boost::core::string_view seg1 = *it1;
             if (dseg0 == seg1)
             {
                 ++it0;
@@ -1612,6 +1722,39 @@ encoding()
     }
 }
 
+void
+readme_snippets()
+{
+    // Parse a URL. This allocates no memory. The view
+    // references the character buffer without taking ownership.
+    //
+    url_view uv( "https://www.example.com/path/to/file.txt?id=1001&name=John%20Doe&results=full" );
+
+    // Print the query parameters with percent-decoding applied
+    //
+    for( auto v : uv.params() )
+    {
+        std::cout << v.key << "=" << v.value << " ";
+    }
+
+    // Prints: id=1001 name=John Doe results=full
+
+    // Create a modifiable copy of `uv`, with ownership of the buffer
+    //
+    url u = uv;
+
+    // Change some elements in the URL
+    //
+    u.set_scheme( "http" )
+        .set_encoded_host( "boost.org" )
+        .set_encoded_path( "/index.htm" )
+        .remove_query()
+        .remove_fragment()
+        .params().append( {"key", "value"} );
+
+    std::cout << u;
+}
+
 //[snippet_using_static_pool_1
 // VFALCO NOPE
 //]
@@ -1629,6 +1772,7 @@ public:
         ignore_unused(&using_urls);
         // parsing_urls();
         parsing_components();
+        formatting_components();
         ignore_unused(&parsing_scheme);
         ignore_unused(&parsing_authority);
         ignore_unused(&parsing_path);
@@ -1640,6 +1784,7 @@ public:
         ignore_unused(&modifying_path);
         normalizing();
         encoding();
+        ignore_unused(&readme_snippets);
 
         BOOST_TEST_PASS();
     }
