@@ -83,13 +83,40 @@ public:
             BOOST_TEST_EQ(u1.data(), u2.data());
         }
 
-        // url_view(string_view)
+        // url_view(core::string_view)
         {
             BOOST_TEST_NO_THROW(url_view(
                 "http://example.com/path/to/file.txt?#"));
             BOOST_TEST_THROWS(url_view("{}"),
                 std::exception);
         }
+
+        // url_view(core::string_view) no ambiguous
+        {
+            core::string_view s = "";
+            BOOST_TEST_NO_THROW(url_view( s ));
+        }
+
+        // implicit url_view(core::string_view)
+        {
+            auto const f = []( url_view ) {};
+            f( "x" );
+        }
+
+        // implicit url_view(String)
+        {
+            auto const f = []( url_view ) {};
+            f( "x" );
+        }
+
+        // issue #756
+        {
+            url u("http://example.com");
+            auto foo = [&u](url_view uv) {
+                BOOST_TEST(uv.buffer().data() == u.buffer().data());
+            };
+            foo(u);
+        }
     }
 
     void
@@ -113,7 +140,7 @@ public:
 
         // data()
         {
-            string_view s = "/index.htm";
+            core::string_view s = "/index.htm";
             url_view u(s);
             BOOST_TEST_NE(u.data(), nullptr);
             BOOST_TEST_EQ(u.data(), s.data());
@@ -121,10 +148,10 @@ public:
 
         // string()
         {
-            string_view s = "/index.htm";
+            core::string_view s = "/index.htm";
             url_view u = parse_relative_ref(s).value();
-            BOOST_TEST_EQ(u.buffer(), s);
-            BOOST_TEST_EQ(u.buffer().data(), s.data());
+            BOOST_TEST_EQ(u, s);
+            BOOST_TEST_EQ(u.data(), s.data());
         }
 
         // persist()
@@ -145,17 +172,24 @@ public:
             // becomes invalid, but sp remains valid.
         }
         }
+
+        // operator core::string_view()
+        {
+            auto const f = []( core::string_view ) {};
+            f( url_view("x") );
+        }
+
     }
 
     void
     testScheme()
     {
         auto const check = [](
-            string_view s,
+            core::string_view s,
             char const* m,
             scheme id)
         {
-            result<url_view> r =
+            system::result<url_view> r =
                 parse_uri_reference(s);
             if(! BOOST_TEST(r))
                 return;
@@ -163,7 +197,7 @@ public:
             if(m)
             {
                 BOOST_TEST(u.scheme() ==
-                           string_view(m));
+                           core::string_view(m));
                 BOOST_TEST(
                     u.scheme_id() == id);
             }
@@ -176,9 +210,9 @@ public:
         };
 
         auto const bad = [](
-            string_view s)
+            core::string_view s)
         {
-            result<url_view> r =
+            system::result<url_view> r =
                 parse_uri_reference(s);
             BOOST_TEST(r.has_error());
         };
@@ -202,7 +236,7 @@ public:
     testAuthority()
     {
         auto const no =
-            [](string_view s)
+            [](core::string_view s)
         {
             BOOST_TEST_NO_THROW([s]
             {
@@ -211,7 +245,7 @@ public:
             }());
         };
         auto const yes =
-            [](string_view s, string_view m)
+            [](core::string_view s, core::string_view m)
         {
             //BOOST_TEST_NO_THROW(
             //[&]
@@ -256,7 +290,7 @@ public:
     testUserinfo()
     {
         auto const no =
-            [](string_view s)
+            [](core::string_view s)
         {
             BOOST_TEST_NO_THROW(
             [s]{
@@ -265,9 +299,9 @@ public:
             }());
         };
         auto const yes =
-            []( string_view s,
-                string_view m1,
-                string_view m2)
+            []( core::string_view s,
+                core::string_view m1,
+                core::string_view m2)
         {
             BOOST_TEST_NO_THROW(
             [&]{
@@ -516,6 +550,16 @@ public:
             BOOST_TEST_EQ(u.authority().port(), "65536");
             BOOST_TEST_EQ(u.authority().port_number(), 0);
         }
+        // issue 666
+        {
+            url_view u("http://:233337");
+            BOOST_TEST(u.has_port());
+            BOOST_TEST_EQ(u.port(), "233337");
+            BOOST_TEST_EQ(u.port_number(), 0);
+            BOOST_TEST(u.authority().has_port());
+            BOOST_TEST_EQ(u.authority().port(), "233337");
+            BOOST_TEST_EQ(u.authority().port_number(), 0);
+        }
     }
 
     void
@@ -556,6 +600,24 @@ public:
             BOOST_TEST(u.authority().encoded_host_and_port() ==
                 "xyz:99999");
         }
+        {
+            // zone_id
+            auto check = [](core::string_view u0, core::string_view zone_id)
+            {
+                auto ru = parse_uri(u0);
+                BOOST_TEST(ru.has_value());
+                auto const& u = *ru;
+                BOOST_TEST_EQ(u.encoded_zone_id(), zone_id);
+                pct_string_view ps{zone_id};
+                BOOST_TEST_EQ(u.zone_id(), ps.decode());
+            };
+            check("http://[fe80::1%25eth0]/", "eth0");
+            check("http://[2001:db8::1%25eth1]/index.html", "eth1");
+            check("ftp://[fe80::2%25wlan0]:8080/files/", "wlan0");
+            check("ftp://[fe80::2]:8080/files/", "");
+            check("ftp://a.com/files/", "");
+            BOOST_TEST_NOT(parse_uri("http://[fe80::1%25]/").has_value());
+        }
     }
 
     void
@@ -664,11 +726,11 @@ public:
     testFragment()
     {
         auto const check = [](
-            string_view s,
+            core::string_view s,
             char const* encoded,
-            string_view plain)
+            core::string_view plain)
         {
-            result<url_view> r =
+            system::result<url_view> r =
                 parse_uri_reference(s);
             if(! BOOST_TEST(r))
                 return;
@@ -679,7 +741,7 @@ public:
                     u.has_fragment());
                 BOOST_TEST(
                     u.encoded_fragment() ==
-                        string_view(encoded));
+                        core::string_view(encoded));
                 BOOST_TEST_EQ(
                     u.fragment(), plain);
             }
@@ -690,9 +752,9 @@ public:
             }
         };
 
-        auto const bad = [](string_view s)
+        auto const bad = [](core::string_view s)
         {
-            result<url_view> r =
+            system::result<url_view> r =
                 parse_uri_reference(s);
             BOOST_TEST(r.has_error());
         };
@@ -707,6 +769,7 @@ public:
             "#/?:@!$&'()*+,;=",
             "/?:@!$&'()*+,;=",
             "/?:@!$&'()*+,;=");
+        check("##f", "#f", "#f");
 
         bad("#%%");
 
@@ -726,7 +789,7 @@ public:
     {
         // parse_absolute_uri
         {
-            result<url_view> r;
+            system::result<url_view> r;
 
             r = parse_absolute_uri(
                 "http://user:pass@example.com:443/path/to/file.txt?q");
@@ -741,7 +804,7 @@ public:
 
         // parse_uri
         {
-            result<url_view> r;
+            system::result<url_view> r;
 
             r = parse_uri(
                 "http://user:pass@example.com:443/path/to/file.txt?q#f");
@@ -752,11 +815,14 @@ public:
             BOOST_TEST(r.has_error());
             BOOST_TEST_THROWS(
                 r.value(), std::exception);
+
+            BOOST_TEST(parse_uri(
+                "http://example.com:a").has_error());
         }
 
         // parse_relative_ref
         {
-            result<url_view> r;
+            system::result<url_view> r;
 
             r = parse_relative_ref(
                 "//example.com/path/to/file.txt?q#f");
@@ -771,7 +837,7 @@ public:
 
         // parse_uri_reference
         {
-            result<url_view> r;
+            system::result<url_view> r;
 
             r = parse_uri_reference(
                 "http://user:pass@example.com:443/path/to/file.txt?q#f");
@@ -815,7 +881,7 @@ public:
     testRelativePart()
     {
         auto const ok = [](
-            string_view s)
+            core::string_view s)
         {
             BOOST_TEST_NO_THROW(
                 parse_relative_ref(s).value());
@@ -944,7 +1010,7 @@ public:
     ignore_unused(u);
         }
         {
-    result< url_view > rv = parse_uri_reference( "https://www.example.com/index.htm?text=none#a1" );
+    system::result< url_view > rv = parse_uri_reference( "https://www.example.com/index.htm?text=none#a1" );
 
     ignore_unused(rv);
         }
@@ -956,7 +1022,7 @@ public:
         ignore_unused(u);
         }
 
-        // url_view(string_view)
+        // url_view(core::string_view)
         {
         url_view u( "http://www.example.com/index.htm" );
 
