@@ -13,13 +13,26 @@
     license reproduced at the end of this file.
 =============================================================================*/
 
-// Tests error reporting: illegal #if expressions.
+// Tests the #line directive.
 
-// 14.8: type cast is disallowed.
-//       Evaluated as: (0)0x8000, constant expression error.
-//E t_6_013.cpp(21): error: ill formed preprocessor expression: (0)0x8000 < 0
-#if (int)0x8000 < 0
-#endif
+// 7.1: Line number and filename.
+//R #line 1234 "cpp"
+#line 1234 "cpp"
+__LINE__                //R 1234 
+__FILE__                //R "cpp" 
+
+// 7.2: Filename argument is optional.
+//R #line 2345 "cpp"
+#line 2345
+__LINE__                //R 2345 
+__FILE__                //R "cpp" 
+
+// 7.3: Argument with macro.
+//R #line 1234 "t_5_008.cpp"
+#define LINE_AND_FILENAME 1234 "t_5_008.cpp"
+#line LINE_AND_FILENAME
+__LINE__                //R 1234 
+__FILE__                //R "t_5_008.cpp" 
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
